﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BeegraffCorp.Data;
using System.Xml.Serialization;
using System.IO;

namespace BeegraffCorp.BLL
{
    public class DepartmentsManager : IEntitiesManager<Department>
    {
        private string _dataFile;

        public DepartmentsManager(string dataFile)
        {
            _dataFile = dataFile;
        }

        #region IEntitiesManager<DepartmentsManager> Members

        public List<Department> SelectAll()
        {
            if (!File.Exists(_dataFile))
                return new List<Department>();

            XmlSerializer serializer = new XmlSerializer(typeof(List<Department>));
            using (FileStream fs = new FileStream(_dataFile, FileMode.Open, FileAccess.Read))
            {
                return (List<Department>)serializer.Deserialize(fs);
            }
        }

        public void Save(Department entity)
        {
            if (entity == null)
                return;

            var departments = SelectAll();
            if (entity.Id == -1)
            {
                if (departments.Any())
                {
                    var lastDepartment = departments.OrderBy(d => d.Id).LastOrDefault();
                    if (lastDepartment != null)
                    {
                        entity.Id = lastDepartment.Id + 1;
                    }
                }
                else
                {
                    entity.Id = 0;
                }

                departments.Add(entity);
            }
            else
            {
                var dep = departments.FirstOrDefault(d => d.Id == entity.Id);
                dep.Name = entity.Name;
            }

            SerializeList(departments);
        }

        public void Delete(Department entity)
        {
            if (entity == null)
                return;

            var departments = SelectAll();
            if (departments.Any())
            {
                var department = departments.FirstOrDefault(d => d.Id == entity.Id);
                if (department != null)
                {
                    departments.Remove(department);
                    SerializeList(departments);
                }
            }
        }

        #endregion

        private void SerializeList(List<Department> departments)
        {
            if (File.Exists(_dataFile))
                File.Delete(_dataFile);

            XmlSerializer serializer = new XmlSerializer(typeof(List<Department>));
            using(FileStream fs = new FileStream(_dataFile, FileMode.OpenOrCreate, FileAccess.Write))
            {
                serializer.Serialize(fs, departments);
            }
        }
    }
}
