﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BeegraffCorp.Data;
using System.IO;
using System.Xml.Serialization;

namespace BeegraffCorp.BLL
{
    public class EmployeesManager : IEntitiesManager<Employee>
    {
        private string _dataFile;

        public EmployeesManager(string dataFile)
        {
            _dataFile = dataFile;
        }

        #region IEntitiesManager<EmployeesManager> Members

        public List<Employee> SelectAll()
        {
            if (!File.Exists(_dataFile))
                return new List<Employee>();

            XmlSerializer serializer = new XmlSerializer(typeof(List<Employee>));
            using (FileStream fs = new FileStream(_dataFile, FileMode.Open, FileAccess.Read))
            {
                return (List<Employee>)serializer.Deserialize(fs);
            }
        }

        public void Save(Employee entity)
        {
            if (entity == null)
                return;

            var employees = SelectAll();
            if (entity.EmployeeId == -1)
            {
                if (employees.Any())
                {
                    var lastEmployee = employees.OrderBy(e => e.EmployeeId).LastOrDefault();
                    if (lastEmployee != null)
                    {
                        entity.EmployeeId = lastEmployee.EmployeeId + 1;
                    }
                }
                else
                {
                    entity.EmployeeId = 0;
                }

                employees.Add(entity);
            }
            else
            {
                var emp = employees.FirstOrDefault(e => e.EmployeeId == entity.EmployeeId);
                emp.Name = entity.Name;
                emp.FirstName = entity.FirstName;
                emp.Birthdate = entity.Birthdate;
                emp.DepartmentId = entity.DepartmentId;
                emp.IsManager = entity.IsManager;
                emp.ManagerId = entity.ManagerId;
            }

            SerializeList(employees);
        }

        public void Delete(Employee entity)
        {
            if (entity == null)
                return;

            var employees = SelectAll();
            if (employees.Any())
            {
                var employee = employees.FirstOrDefault(e => e.EmployeeId == entity.EmployeeId);
                if (employee != null)
                {
                    employees.Remove(employee);
                    SerializeList(employees);
                }
            }
        }

        #endregion

        private void SerializeList(List<Employee> employees)
        {
            if (File.Exists(_dataFile))
            {
                File.Delete(_dataFile);
            }

            XmlSerializer serializer = new XmlSerializer(typeof(List<Employee>));
            using(FileStream fs = new FileStream(_dataFile, FileMode.OpenOrCreate, FileAccess.Write))
            {
                serializer.Serialize(fs, employees);
            }
        }
    }
}
