﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using BeegraffCorp.BLL;
using System.ComponentModel;
using BeegraffCorp.Data;
using System.Windows.Data;
using System.Windows.Input;
using BeegraffCorp.Commands;
using System.Windows;

namespace BeegraffCorp.ViewModels
{
    public class DepartmentsManagementViewModel : ViewModelBase
    {
        private readonly ObservableCollection<Department> _departments = new ObservableCollection<Department>();

        public ObservableCollection<Department> Departments
        {
            get { return _departments; }
        } 

        private DepartmentsManager _departmentsManager;
        private ICollectionView _departmentsCollectionView;

        public DepartmentsManagementViewModel()
        {

            _departmentsCollectionView = CollectionViewSource.GetDefaultView(Departments);
            _departmentsCollectionView.CurrentChanged += OnEmployeesCollectionViewCurrentChanged;

            _departmentsManager = new DepartmentsManager("Data/departments.xml");

            Refresh();
        }

        public Department CurrentDepartment
        {
            get
            {
                if (_departmentsCollectionView != null)
                    return _departmentsCollectionView.CurrentItem as Department;

                return null;
            }
        }

        private void OnEmployeesCollectionViewCurrentChanged(object sender, EventArgs e)
        {
            OnPropertyChanged("CurrentDepartment");
            IsEditMode = _departmentsCollectionView.CurrentItem != null;
        }

        private bool _isEditMode;

        public bool IsEditMode
        {
            get { return _isEditMode; }
            set 
            { 
                _isEditMode = value;
                OnPropertyChanged("IsEditMode");
            }
        }

        private ICommand _deleteCommand;

        public ICommand DeleteCommand
        {
            get 
            {
                if (_deleteCommand == null)
                {
                    _deleteCommand = new RelayCommand(OnDeleteCommand);
                }
                return _deleteCommand; 
            }
        }

        private ICommand _saveCommand;

        public ICommand SaveCommand
        {
            get
            {
                if (_saveCommand == null)
                {
                    _saveCommand = new RelayCommand(OnSaveCommand);
                }
                return _saveCommand;
            }
        }

        private ICommand _newCommand;

        public ICommand NewCommand
        {
            get
            {
                if (_newCommand == null)
                {
                    _newCommand = new RelayCommand(OnNewCommand);
                }
                return _newCommand;
            }
        }

        private void OnDeleteCommand(object param)
        {
            if (CurrentDepartment != null && MessageBox.Show("Are you sure to delete this department ?", "Question", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                _departmentsManager.Delete(CurrentDepartment);
                Departments.Remove(CurrentDepartment);
            }
        }

        private void OnSaveCommand(object param)
        {
            _departmentsManager.Save(CurrentDepartment);
            IsEditMode = false;
            Refresh();
        }

        private void OnNewCommand(object param)
        {
            Department newDep = new Department { Id = -1 };
            Departments.Add(newDep);
            _departmentsCollectionView.MoveCurrentTo(newDep);
            IsEditMode = true;
        }

        private void Refresh()
        {
            Departments.Clear();
            foreach (var department in _departmentsManager.SelectAll())
            {
                Departments.Add(department);
            }
        }
    }
}
