﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BeegraffCorp.Data;
using System.Collections.ObjectModel;
using BeegraffCorp.BLL;
using System.ComponentModel;
using System.Windows.Data;
using System.Windows.Input;
using BeegraffCorp.Commands;
using System.Windows;

namespace BeegraffCorp.ViewModels
{
    public class EmployeesManagementViewModel : ViewModelBase
    {
        private readonly ObservableCollection<Employee> _employees = new ObservableCollection<Employee>();

        public ObservableCollection<Employee> Employees
        {
            get { return _employees; }
        }

        private readonly ObservableCollection<Department> _departments = new ObservableCollection<Department>();

        public ObservableCollection<Department> Departments
        {
            get { return _departments; }
        } 

        private EmployeesManager _employeesManager;
        private DepartmentsManager _departmentsManager;
        private ICollectionView _employeesCollectionView;

        public EmployeesManagementViewModel()
        {
            _employeesManager = new EmployeesManager("Data/employees.xml");


            _employeesCollectionView = CollectionViewSource.GetDefaultView(Employees);
            _employeesCollectionView.CurrentChanged += OnEmployeesCollectionViewCurrentChanged;

            _departmentsManager = new DepartmentsManager("Data/departments.xml");
            
            Refresh();
            RefreshDepartments();
        }

        public Employee CurrentEmployee
        {
            get
            {
                if (_employeesCollectionView != null)
                    return _employeesCollectionView.CurrentItem as Employee;

                return null;
            }
        }

        private void OnEmployeesCollectionViewCurrentChanged(object sender, EventArgs e)
        {
            OnPropertyChanged("CurrentEmployee");
            IsEditMode = _employeesCollectionView.CurrentItem != null;
        }

        private bool _isEditMode;

        public bool IsEditMode
        {
            get { return _isEditMode; }
            set
            {
                _isEditMode = value;
                OnPropertyChanged("IsEditMode");
            }
        }

        private ICommand _deleteCommand;

        public ICommand DeleteCommand
        {
            get 
            {
                if (_deleteCommand == null)
                {
                    _deleteCommand = new RelayCommand(OnDeleteCommand);
                }
                return _deleteCommand; 
            }
        }

        private ICommand _saveCommand;

        public ICommand SaveCommand
        {
            get
            {
                if (_saveCommand == null)
                {
                    _saveCommand = new RelayCommand(OnSaveCommand);
                }
                return _saveCommand;
            }
        }

        private ICommand _newCommand;

        public ICommand NewCommand
        {
            get
            {
                if (_newCommand == null)
                {
                    _newCommand = new RelayCommand(OnNewCommand);
                }
                return _newCommand;
            }
        }

        private void OnDeleteCommand(object param)
        {
            if (CurrentEmployee != null && MessageBox.Show("Are you sure to delete this employee ?", "Question", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                _employeesManager.Delete(CurrentEmployee);
                Employees.Remove(CurrentEmployee);
            }
        }

        private void OnSaveCommand(object param)
        {
            _employeesManager.Save(CurrentEmployee);
            Refresh();
        }

        private void OnNewCommand(object param)
        {
            Employee newEmp = new Employee { EmployeeId = -1, Birthdate = new DateTime(1980, 1, 1) };
            Employees.Add(newEmp);
            _employeesCollectionView.MoveCurrentTo(newEmp);
        }

        private void Refresh()
        {
            Employees.Clear();
            foreach (var employee in _employeesManager.SelectAll())
                Employees.Add(employee);
        }

        public void RefreshDepartments()
        {
            Departments.Clear();
            foreach (var department in _departmentsManager.SelectAll())
                Departments.Add(department);
        }
    }
}
